@@ -1,33 +1,32 @@
 """Ezviz API."""
 import hashlib
 import logging
-from random import randint
 from uuid import uuid4
 
 import requests
 from pyezviz.camera import EzvizCamera
 from pyezviz.constants import DefenseModeType, DeviceCatagories
 
-API_BASE_TLD = ".ezvizlife.com"
-API_ENDPOINT_AUTH = "/doLogin"
 API_ENDPOINT_CLOUDDEVICES = "/api/cloud/v2/cloudDevices/getAll"
 API_ENDPOINT_PAGELIST = "/v3/userdevices/v1/devices/pagelist"
 API_ENDPOINT_DEVICES = "/v3/devices/"
 API_ENDPOINT_LOGIN = "/v3/users/login/v5"
+API_ENDPOINT_REFRESH_SESSION_ID = "/v3/apigateway/login"
 API_ENDPOINT_SWITCH_STATUS = "/switchStatus"
 API_ENDPOINT_PTZCONTROL = "/ptzControl"
 API_ENDPOINT_ALARM_SOUND = "/alarm/sound"
-API_ENDPOINT_SET_DEFENCE = "/camera/cameraAction!enableDefence.action"
+API_ENDPOINT_SET_DEFENCE = "/camera/cameraAction!enableDefence.action"  # Needs changing
 API_ENDPOINT_DETECTION_SENSIBILITY = "/api/device/configAlgorithm"
 API_ENDPOINT_DETECTION_SENSIBILITY_GET = "/api/device/queryAlgorithmConfig"
 API_ENDPOINT_ALARMINFO_GET = "/v3/alarms/v2/advanced"
-API_ENDPOINT_CHECKLOGIN = "/user/user/userAction!checkLoginInfo.action"
 API_ENDPOINT_SET_DEFENCE_SCHEDULE = "/api/device/defence/plan2"
 API_ENDPOINT_SWITCH_DEFENCE_MODE = "/v3/userdevices/v1/group/switchDefenceMode"
 API_ENDPOINT_SWITCH_SOUND_ALARM = "/sendAlarm"
 
 DEFAULT_TIMEOUT = 25
 MAX_RETRIES = 3
+FEATURE_CODE = "c22cb01f8cb83351422d82fad59c8e4e"
+# token = {"sessionId": None, "refreshSessionId": None}
 
 
 class PyEzvizError(Exception):
@@ -39,94 +38,26 @@ class EzvizClient:
 
     def __init__(
         self,
-        account,
-        password,
-        region="eu",
+        account=None,
+        password=None,
+        url="apiieu.ezvizlife.com",
         timeout=DEFAULT_TIMEOUT,
+        token=None,
     ):
         """Initialize the client object."""
         self.account = account
         self.password = password
         self._session = None
+        self._token = token or {"session_id": None, "rf_session_id": None}
         self._timeout = timeout
-        self._csrf_token = 0
-        self.api_domain = "apii" + region
-        self.auth_domain = region + "auth"
+        self.api_uri = url
 
     def _login(self):
-        """Login to Ezviz' API."""
+        """Login to Ezviz API."""
 
-        # Ezviz API sends md5 of password
-        temp = hashlib.md5()
-        temp.update(self.password.encode("utf-8"))
-        md5pass = temp.hexdigest()
-        randommath = randint(1000000000000000000, 9999999999999999999)
-        payload = {
-            "account": self.account,
-            "password": md5pass,
-            "from": "4e4148ba90184a7cbd81",
-            "r": randommath,
-            "returnUrl": "plugin",
-            "host": self.api_domain + API_BASE_TLD,
-        }
-
-        try:
-            req = self._session.post(
-                "https://" + self.auth_domain + API_BASE_TLD + API_ENDPOINT_AUTH,
-                allow_redirects=False,
-                data=payload,
-                timeout=self._timeout,
-            )
-
-            req.raise_for_status()
-
-        except requests.HTTPError as err:
-            raise PyEzvizError("Can not login to API") from err
-
-        try:
-            json_result = req.json()
-
-        except ValueError as err:
-            raise PyEzvizError("Can't decode response") from err
-
-        if self._session.cookies.get("REDIRECTCOOKIE", domain=API_BASE_TLD):
-            print("Your region is incorrect!")
-            self._login_api()
-
-        if json_result["retcode"] == "1001":
-            raise PyEzvizError("Incorrect login details")
-
-        if json_result["retcode"] == "1002":
-            raise PyEzvizError("Login error: Captcha required")
-
-        if json_result["retcode"] == "1005":
-            raise PyEzvizError("Login error: Incorrect Captcha code")
-
-        if req.status_code != 200:
-            raise PyEzvizError(
-                f"Login error: Please check your username/password: {req.text} "
-            )
-
-        # Update cookie with JS Session ID that is generated by website.
-        try:
-            req = self._session.get(
-                "https://" + self.api_domain + API_BASE_TLD + "/check_plugin.jsp",
-                timeout=self._timeout,
-            )
-
-            req.raise_for_status()
-
-            self._csrf_token = self._session.cookies.get(
-                "AS_SessionID", domain=self.api_domain + API_BASE_TLD
-            )
-
-        except requests.HTTPError as err:
-            raise PyEzvizError("Can not login to API") from err
-
-        return True
-
-    def _login_api(self):
-        """Login to Ezviz' API."""
+        # Region code to url.
+        if len(self.api_uri.split(".")) == 1:
+            self.api_uri = "apii" + self.api_uri + ".ezvizlife.com"
 
         # Ezviz API sends md5 of password
         temp = hashlib.md5()
@@ -135,12 +66,12 @@ class EzvizClient:
         payload = {
             "account": self.account,
             "password": md5pass,
-            "featureCode": "d9edb711e6563e99d366d1a84602f51f",
+            "featureCode": FEATURE_CODE,
         }
 
         try:
             req = self._session.post(
-                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_LOGIN,
+                "https://" + self.api_uri + API_ENDPOINT_LOGIN,
                 allow_redirects=False,
                 headers={"clientType": "3"},
                 data=payload,
@@ -149,21 +80,44 @@ class EzvizClient:
 
             req.raise_for_status()
 
+        except requests.HTTPError as err:
+            raise requests.HTTPError(err)
+
+        try:
             json_result = req.json()
 
-            if json_result["meta"]["code"] == 1100:
-                region = json_result["loginArea"]["apiDomain"]
-                raise PyEzvizError(f"region url: {region} ")
-
-        except requests.HTTPError as err:
-            raise PyEzvizError("Can not login to API") from err
-
-        if req.status_code != 200:
+        except ValueError as err:
             raise PyEzvizError(
-                f"Login error: Please check your username/password: {req.text} "
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_result["meta"]["code"] == 1100:
+            self.api_uri = json_result["loginArea"]["apiDomain"]
+            print("Region incorrect!")
+            print(f"Your region url: {self.api_uri}")
+            self._session = None
+            return self.login()
+
+        if json_result["meta"]["code"] == 1013:
+            raise PyEzvizError("Incorrect Username.")
+
+        if json_result["meta"]["code"] == 1014:
+            raise PyEzvizError("Incorrect Password.")
+
+        if json_result["meta"]["code"] == 1015:
+            raise PyEzvizError("The user is locked.")
+
+        self._token["session_id"] = str(json_result["loginSession"]["sessionId"])
+        self._token["rf_session_id"] = str(json_result["loginSession"]["rfSessionId"])
+        if not self._token["session_id"]:
+            raise PyEzvizError(
+                f"Login error: Please check your username/password: {req.text}"
             )
 
-        return True
+        return self._token
 
     def _api_get_pagelist(self, page_filter=None, json_key=None, max_retries=0):
         """Get data from pagelist API."""
@@ -176,7 +130,8 @@ class EzvizClient:
 
         try:
             req = self._session.get(
-                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_PAGELIST,
+                "https://" + self.api_uri + API_ENDPOINT_PAGELIST,
+                headers={"sessionId": self._token["session_id"]},
                 params={"filter": page_filter},
                 timeout=self._timeout,
             )
@@ -184,13 +139,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info("Got %s, relogging", str(req.status_code))
                 return self._api_get_pagelist(page_filter, json_key, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         if not req.text:
             raise PyEzvizError("No data")
@@ -209,7 +163,9 @@ class EzvizClient:
         if json_output.get("meta").get("code") != 200:
             # session is wrong, need to relogin
             self.login()
-            logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
+            logging.info(
+                "Json request error, relogging (max retries: %s)", str(max_retries)
+            )
             return self._api_get_pagelist(page_filter, json_key, max_retries + 1)
 
         if json_key is None:
@@ -235,10 +191,8 @@ class EzvizClient:
 
         try:
             req = self._session.get(
-                "https://"
-                + self.api_domain
-                + API_BASE_TLD
-                + API_ENDPOINT_ALARMINFO_GET,
+                "https://" + self.api_uri + API_ENDPOINT_ALARMINFO_GET,
+                headers={"sessionId": self._token["session_id"]},
                 params={
                     "deviceSerials": serial,
                     "queryType": -1,
@@ -251,15 +205,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info(
-                    "Got 302 or 401, relogging (max retries: %s)", str(max_retries)
-                )
                 return self.get_alarminfo(serial, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         if req.text == "":
             raise PyEzvizError("No data")
@@ -279,17 +230,19 @@ class EzvizClient:
 
     def _switch_status(self, serial, status_type, enable, max_retries=0):
         """Switch status on a device."""
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
 
         try:
             req = self._session.put(
                 "https://"
-                + self.api_domain
-                + API_BASE_TLD
+                + self.api_uri
                 + API_ENDPOINT_DEVICES
                 + serial
                 + "/1/1/"
                 + str(status_type)
                 + API_ENDPOINT_SWITCH_STATUS,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "enable": enable,
                     "serial": serial,
@@ -302,15 +255,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info(
-                    "Got 504 or 401, relogging (max retries: %s)", str(max_retries)
-                )
                 return self._switch_status(serial, status_type, enable, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             json_output = req.json()
@@ -332,16 +282,18 @@ class EzvizClient:
 
     def sound_alarm(self, serial, enable=1, max_retries=0):
         """Sound alarm on a device."""
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
 
         try:
             req = self._session.put(
                 "https://"
-                + self.api_domain
-                + API_BASE_TLD
+                + self.api_uri
                 + API_ENDPOINT_DEVICES
                 + serial
                 + "/0"
                 + API_ENDPOINT_SWITCH_SOUND_ALARM,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "enable": enable,
                 },
@@ -351,15 +303,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info(
-                    "Got 504 or 401, relogging (max retries: %s)", str(max_retries)
-                )
                 return self.sound_alarm(serial, enable, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             json_output = req.json()
@@ -516,8 +465,7 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_domain
-                + API_BASE_TLD
+                + self.api_uri
                 + API_ENDPOINT_DEVICES
                 + serial
                 + API_ENDPOINT_PTZCONTROL,
@@ -529,14 +477,14 @@ class EzvizClient:
                     "uuid": str(uuid4()),
                     "serial": serial,
                 },
-                headers={"clientType": "1"},
+                headers={"sessionId": self._token["session_id"], "clientType": "1"},
                 timeout=self._timeout,
             )
 
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         return req.text
 
@@ -544,33 +492,43 @@ class EzvizClient:
         """Set http session."""
         if self._session is None:
             self._session = requests.session()
-            self._login()
+            return self._login()
 
-        try:
-            req = self._session.get(
-                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_CHECKLOGIN,
-                allow_redirects=False,
-                timeout=self._timeout,
+        if self._token["session_id"] and self._token["rf_session_id"]:
+            try:
+                req = self._session.put(
+                    "https://" + self.api_uri + API_ENDPOINT_REFRESH_SESSION_ID,
+                    data={
+                        "refreshSessionId": self._token["rf_session_id"],
+                        "featureCode": FEATURE_CODE,
+                    },
+                    headers={"sessionId": self._token["session_id"]},
+                    timeout=self._timeout,
+                )
+                req.raise_for_status()
+
+            except requests.HTTPError as err:
+                raise requests.HTTPError(err)
+
+            try:
+                json_result = req.json()
+
+            except ValueError as err:
+                raise PyEzvizError(
+                    "Impossible to decode response: "
+                    + str(err)
+                    + "\nResponse was: "
+                    + str(req.text)
+                ) from err
+
+            self._token["session_id"] = str(json_result["sessionInfo"]["sessionId"])
+            self._token["rf_session_id"] = str(
+                json_result["sessionInfo"]["refreshSessionId"]
             )
+            if not self._token["session_id"]:
+                raise PyEzvizError(f"Relogin required: {req.text}")
 
-            req.raise_for_status()
-
-        except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
-                self._login()
-
-            raise PyEzvizError(
-                "Could not access Ezviz login check API: " + str(err)
-            ) from err
-
-        try:
-            response_json = req.json()
-
-        except ValueError as err:
-            raise PyEzvizError("Can't decode response" + str(err)) from err
-
-        if response_json["success"] != "success":
-            self._login()
+            return self._token
 
         return True
 
@@ -581,7 +539,8 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://" + self.api_domain + API_BASE_TLD + API_ENDPOINT_SET_DEFENCE,
+                "https://" + self.api_uri + API_ENDPOINT_SET_DEFENCE,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "deviceSerial": serial,
                     "defenceType": "Global",
@@ -594,15 +553,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info(
-                    "Got 504 or 401, relogging (max retries: %s)", str(max_retries)
-                )
                 return self.data_report(serial, enable, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             json_output = req.json()
@@ -617,7 +573,7 @@ class EzvizClient:
 
         if json_output.get("resultCode") != "0":
             raise PyEzvizError(
-                f"Could not set detection sensibility level: Got {req.status_code} : {req.text})"
+                f"Could not set alarm notification: Got {req.status_code} : {req.text})"
             )
 
         return True
@@ -638,10 +594,8 @@ class EzvizClient:
         )
         try:
             req = self._session.post(
-                "https://"
-                + self.api_domain
-                + API_BASE_TLD
-                + API_ENDPOINT_SET_DEFENCE_SCHEDULE,
+                "https://" + self.api_uri + API_ENDPOINT_SET_DEFENCE_SCHEDULE,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "devTimingPlan": schedulestring,
                 },
@@ -651,15 +605,14 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info("Error, got status code: %s", req.status_code)
                 return self.api_set_defence_schedule(
                     serial, schedule, enable, max_retries + 1
                 )
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             json_output = req.json()
@@ -686,10 +639,8 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://"
-                + self.api_domain
-                + API_BASE_TLD
-                + API_ENDPOINT_SWITCH_DEFENCE_MODE,
+                "https://" + self.api_uri + API_ENDPOINT_SWITCH_DEFENCE_MODE,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "groupId": -1,
                     "mode": mode,
@@ -700,13 +651,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                logging.info("Error, got status code: %s", req.status_code)
                 return self.api_set_defence_mode(mode, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             json_output = req.json()
@@ -738,10 +688,8 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://"
-                + self.api_domain
-                + API_BASE_TLD
-                + API_ENDPOINT_DETECTION_SENSIBILITY,
+                "https://" + self.api_uri + API_ENDPOINT_DETECTION_SENSIBILITY,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "subSerial": serial,
                     "type": type_value,
@@ -754,15 +702,14 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to re-log-in
                 self.login()
-                logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
                 return self.detection_sensibility(
                     serial, sensibility, type_value, max_retries + 1
                 )
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             response_json = req.json()
@@ -782,10 +729,8 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://"
-                + self.api_domain
-                + API_BASE_TLD
-                + API_ENDPOINT_DETECTION_SENSIBILITY_GET,
+                "https://" + self.api_uri + API_ENDPOINT_DETECTION_SENSIBILITY_GET,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "subSerial": serial,
                 },
@@ -795,15 +740,14 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
-                # session is wrong, need to re-log-in
+            if err.response.status_code == 401:
+                # session is wrong, need to re-log-in.
                 self.login()
-                logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
                 return self.get_detection_sensibility(
                     serial, type_value, max_retries + 1
                 )
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         try:
             response_json = req.json()
@@ -819,7 +763,7 @@ class EzvizClient:
                 if idx["type"] == type_value:
                     return idx["value"]
 
-        return "Unkown"
+        return "Unknown"
 
     # soundtype: 0 = normal, 1 = intensive, 2 = disabled ... don't ask me why...
     def alarm_sound(self, serial, sound_type, enable=1, max_retries=0):
@@ -835,11 +779,11 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_domain
-                + API_BASE_TLD
+                + self.api_uri
                 + API_ENDPOINT_DEVICES
                 + serial
                 + API_ENDPOINT_ALARM_SOUND,
+                headers={"sessionId": self._token["session_id"]},
                 data={
                     "enable": enable,
                     "soundType": sound_type,
@@ -852,13 +796,12 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            if err.response.status_code == 401 or 504:
+            if err.response.status_code == 401:
                 # session is wrong, need to re-log-in
                 self.login()
-                logging.info("Got 401, relogging (max retries: %s)", str(max_retries))
                 return self.alarm_sound(serial, sound_type, enable, max_retries + 1)
 
-            raise PyEzvizError("Could not access Ezviz' API: " + str(err)) from err
+            raise requests.HTTPError(err)
 
         return True
 
@@ -920,5 +863,6 @@ class EzvizClient:
 
     def close_session(self):
         """Close current session."""
-        self._session.close()
-        self._session = None
+        if self._session:
+            self._session.close()
+            self._session = None
